@@ -5,8 +5,10 @@
 #include <fcntl.h>
 #include <qpdf/QUtil.hh>
 #include <qpdf/PointerHolder.hh>
+#include <qpdf/QPDFSystemError.hh>
 #include <string.h>
 #include <limits.h>
+#include <assert.h>
 
 #ifdef _WIN32
 # include <io.h>
@@ -26,7 +28,7 @@ void test_to_number(char const* str, int_T wanted, bool error,
         result = fn(str);
         worked = (wanted == result);
     }
-    catch (std::runtime_error)
+    catch (std::runtime_error const&)
     {
         threw = true;
     }
@@ -137,9 +139,10 @@ void fopen_wrapper_test()
 	std::cout << "after fopen" << std::endl;
 	(void) fclose(f);
     }
-    catch (std::runtime_error& s)
+    catch (QPDFSystemError& s)
     {
 	std::cout << "exception: " << s.what() << std::endl;
+        assert(s.getErrno() != 0);
     }
 }
 
@@ -247,7 +250,7 @@ void same_file_test()
         fclose(QUtil::safe_fopen("qutil.out", "r"));
         fclose(QUtil::safe_fopen("other-file", "r"));
     }
-    catch (std::exception)
+    catch (std::exception const&)
     {
         std::cout << "same_file_test expects to have qutil.out and other-file"
             " exist in the current directory\n";
