@@ -8,6 +8,7 @@
 #include <qpdf/QUtil.hh>
 #include <qpdf/QTC.hh>
 #include <qpdf/ClosedFileInputSource.hh>
+#include <qpdf/FileInputSource.hh>
 #include <qpdf/Pl_StdioFile.hh>
 #include <qpdf/Pl_Discard.hh>
 #include <qpdf/PointerHolder.hh>
@@ -61,6 +62,7 @@ struct Options
         split_pages(0),
         verbose(false),
         progress(false),
+        suppress_warnings(false),
         copy_encryption(false),
         encryption_file(0),
         encryption_file_password(0),
@@ -94,6 +96,8 @@ struct Options
         qdf_mode(false),
         preserve_unreferenced_objects(false),
         preserve_unreferenced_page_resources(false),
+        keep_files_open(true),
+        keep_files_open_set(false),
         newline_before_endstream(false),
         coalesce_contents(false),
         show_npages(false),
@@ -125,6 +129,7 @@ struct Options
     int split_pages;
     bool verbose;
     bool progress;
+    bool suppress_warnings;
     bool copy_encryption;
     char const* encryption_file;
     char const* encryption_file_password;
@@ -160,6 +165,8 @@ struct Options
     bool qdf_mode;
     bool preserve_unreferenced_objects;
     bool preserve_unreferenced_page_resources;
+    bool keep_files_open;
+    bool keep_files_open_set;
     bool newline_before_endstream;
     std::string linearize_pass1;
     bool coalesce_contents;
@@ -262,6 +269,7 @@ Basic Options\n\
 --password=password     specify a password for accessing encrypted files\n\
 --verbose               provide additional informational output\n\
 --progress              give progress indicators while writing output\n\
+--no-warn               suppress warnings\n\
 --linearize             generated a linearized (web optimized) file\n\
 --copy-encryption=file  copy encryption parameters from specified file\n\
 --encryption-file-password=password\n\
@@ -381,6 +389,7 @@ These options allow pages to be selected from one or more PDF files.\n\
 Whatever file is given as the primary input file is used as the\n\
 starting point, but its pages are replaced with pages as specified.\n\
 \n\
+--keep-files-open=[yn]\n\
 --pages file [ --password=password ] [ page-range ] ... --\n\
 \n\
 For each file that pages should be taken from, specify the file, a\n\
@@ -393,6 +402,13 @@ non-page data (info, outlines, page numbers, etc. are taken from the\n\
 primary input file.  To discard this, use --empty as the primary\n\
 input.\n\
 \n\
+By default, when more than 200 distinct files are specified, qpdf will\n\
+close each file when not being referenced. With 200 files or fewer, all\n\
+files will be kept open at the same time. This behavior can be overridden\n\
+by specifying --keep-files-open=[yn]. Closing and opening files can have\n\
+very high overhead on certain file systems, especially networked file\n\
+systems.\n\
+\n\
 The page range is a set of numbers separated by commas, ranges of\n\
 numbers separated dashes, or combinations of those.  The character\n\
 \"z\" represents the last page.  A number preceded by an \"r\" indicates\n\
@@ -515,8 +531,9 @@ page content stream.  This attempt will be made even if it is not a\n\
 page content stream, in which case it will produce unusable results.\n\
 \n\
 Ordinarily, qpdf exits with a status of 0 on success or a status of 2\n\
-if any errors occurred.  In --check mode, if there were warnings but not\n\
-errors, qpdf exits with a status of 3.\n\
+if any errors occurred.  If there were warnings but not errors, qpdf\n\
+exits with a status of 3. If warnings would have been issued but --no-warn\n\
+was given, an exit status of 3 is still used.\n\
 \n";
 
 void usage(std::string const& msg)
@@ -773,7 +790,7 @@ static std::vector<int> parse_numrange(char const* range, int max,
             }
         }
     }
-    catch (std::runtime_error e)
+    catch (std::runtime_error const& e)
     {
         if (throw_error)
         {
@@ -1372,7 +1389,7 @@ static void parse_rotation_parameter(Options& o, std::string const& parameter)
         parse_numrange(range.c_str(), 0, true);
         range_valid = true;
     }
-    catch (std::runtime_error)
+    catch (std::runtime_error const&)
     {
         // ignore
     }
@@ -1627,6 +1644,23 @@ static void parse_options(int argc, char* argv[], Options& o)
             {
                 o.preserve_unreferenced_page_resources = true;
             }
+            else if (strcmp(arg, "keep-files-open") == 0)
+            {
+                o.keep_files_open_set = true;
+                if (parameter && (strcmp(parameter, "y") == 0))
+                {
+                    o.keep_files_open = true;
+                }
+                else if (parameter && (strcmp(parameter, "n") == 0))
+                {
+                    o.keep_files_open = false;
+                }
+                else
+                {
+                    usage("--keep-files-open must be given as"
+                          " --keep-files-open=[yn]");
+                }
+            }
             else if (strcmp(arg, "newline-before-endstream") == 0)
             {
                 o.newline_before_endstream = true;
@@ -1676,6 +1710,10 @@ static void parse_options(int argc, char* argv[], Options& o)
             {
                 o.progress = true;
             }
+            else if (strcmp(arg, "no-warn") == 0)
+            {
+                o.suppress_warnings = true;
+            }
             else if (strcmp(arg, "deterministic-id") == 0)
             {
                 o.deterministic_id = true;
@@ -1832,6 +1870,10 @@ static void set_qpdf_options(QPDF& pdf, Options& o)
     {
         pdf.setPasswordIsHexKey(true);
     }
+    if (o.suppress_warnings)
+    {
+        pdf.setSuppressWarnings(true);
+    }
 }
 
 static void do_check(QPDF& pdf, Options& o, int& exit_code)
@@ -2101,8 +2143,46 @@ static void handle_page_specs(QPDF& pdf, Options& o,
     // Parse all page specifications and translate them into lists of
     // actual pages.
 
+    if (! o.keep_files_open_set)
+    {
+        // Count the number of distinct files to determine whether we
+        // should keep files open or not. Rather than trying to code
+        // some portable heuristic based on OS limits, just hard-code
+        // this at a given number and allow users to override.
+        std::set<std::string> filenames;
+        for (std::vector<PageSpec>::iterator iter = o.page_specs.begin();
+             iter != o.page_specs.end(); ++iter)
+        {
+            PageSpec& page_spec = *iter;
+            filenames.insert(page_spec.filename);
+        }
+        // NOTE: The number 200 for this threshold is in the help
+        // message and manual and is baked into the test suite.
+        if (filenames.size() > 200)
+        {
+            QTC::TC("qpdf", "qpdf disable keep files open");
+            if (o.verbose)
+            {
+                std::cout << whoami << ": selecting --keep-open-files=n"
+                          << std::endl;
+            }
+            o.keep_files_open = false;
+        }
+        else
+        {
+            if (o.verbose)
+            {
+                std::cout << whoami << ": selecting --keep-open-files=y"
+                          << std::endl;
+            }
+            o.keep_files_open = true;
+            QTC::TC("qpdf", "qpdf don't disable keep files open");
+        }
+    }
+
     // Create a QPDF object for each file that we may take pages from.
     std::map<std::string, QPDF*> page_spec_qpdfs;
+    std::map<std::string, ClosedFileInputSource*> page_spec_cfis;
     page_spec_qpdfs[o.infilename] = &pdf;
     std::vector<QPDFPageData> parsed_specs;
     for (std::vector<PageSpec>::iterator iter = o.page_specs.begin();
@@ -2136,10 +2216,29 @@ static void handle_page_specs(QPDF& pdf, Options& o,
                 std::cout << whoami << ": processing "
                           << page_spec.filename << std::endl;
             }
-            qpdf->processInputSource(
-                new ClosedFileInputSource(
-                    page_spec.filename.c_str()), password);
+            PointerHolder<InputSource> is;
+            ClosedFileInputSource* cis = 0;
+            if (! o.keep_files_open)
+            {
+                QTC::TC("qpdf", "qpdf keep files open n");
+                cis = new ClosedFileInputSource(page_spec.filename.c_str());
+                is = cis;
+                cis->stayOpen(true);
+            }
+            else
+            {
+                QTC::TC("qpdf", "qpdf keep files open y");
+                FileInputSource* fis = new FileInputSource();
+                is = fis;
+                fis->setFilename(page_spec.filename.c_str());
+            }
+            qpdf->processInputSource(is, password);
             page_spec_qpdfs[page_spec.filename] = qpdf;
+            if (cis)
+            {
+                cis->stayOpen(false);
+                page_spec_cfis[page_spec.filename] = cis;
+            }
         }
 
         // Read original pages from the PDF, and parse the page range
@@ -2156,9 +2255,20 @@ static void handle_page_specs(QPDF& pdf, Options& o,
                  page_spec_qpdfs.begin();
              iter != page_spec_qpdfs.end(); ++iter)
         {
+            std::string const& filename = (*iter).first;
+            ClosedFileInputSource* cis = 0;
+            if (page_spec_cfis.count(filename))
+            {
+                cis = page_spec_cfis[filename];
+                cis->stayOpen(true);
+            }
             QPDFPageDocumentHelper dh(*((*iter).second));
             dh.pushInheritedAttributesToPage();
             dh.removeUnreferencedResources();
+            if (cis)
+            {
+                cis->stayOpen(false);
+            }
         }
     }
 
@@ -2204,7 +2314,17 @@ static void handle_page_specs(QPDF& pdf, Options& o,
             // Pages are specified from 1 but numbered from 0 in the
             // vector
             int pageno = *pageno_iter - 1;
+            ClosedFileInputSource* cis = 0;
+            if (page_spec_cfis.count(page_data.filename))
+            {
+                cis = page_spec_cfis[page_data.filename];
+                cis->stayOpen(true);
+            }
             dh.addPage(page_data.orig_pages.at(pageno), false);
+            if (cis)
+            {
+                cis->stayOpen(false);
+            }
             if (page_data.qpdf == &pdf)
             {
                 // This is a page from the original file. Keep track
@@ -2581,9 +2701,14 @@ int main(int argc, char* argv[])
 	}
 	if (! pdf.getWarnings().empty())
 	{
-	    std::cerr << whoami << ": operation succeeded with warnings;"
-		      << " resulting file may have some problems" << std::endl;
-	    exit(EXIT_WARNING);
+            if (! o.suppress_warnings)
+            {
+                std::cerr << whoami << ": operation succeeded with warnings;"
+                          << " resulting file may have some problems"
+                          << std::endl;
+            }
+            // Still exit with warning code even if warnings were suppressed.
+            exit(EXIT_WARNING);
 	}
     }
     catch (std::exception& e)
